/*
 * Copyright (c) 2024, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 */

#ifndef __VIRTNET_DPA_SPINLOCK_H__
#define __VIRTNET_DPA_SPINLOCK_H__

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

struct virtnet_spinlock_s {
	uint32_t locked;        /**< Indication for spinlock lock state. */
};

#define virtnet_spin_init(lock) \
	__atomic_store_n(&((lock)->locked), 0, __ATOMIC_SEQ_CST)
#define virtnet_spin_lock(lock) \
	do { \
		while (__atomic_exchange_n(&((lock)->locked), 1, __ATOMIC_SEQ_CST)) \
			; \
	} while (0)

#define virtnet_spin_unlock(lock) \
	__atomic_store_n(&((lock)->locked), 0, __ATOMIC_SEQ_CST)
#define virtnet_spin_trylock(lock) \
	__atomic_exchange_n(&((lock)->locked), 1, __ATOMIC_SEQ_CST)

#ifdef __cplusplus
}
#endif

#endif
